from .bitstamp import BitstampFetcher
from .cex import CexFetcher
from .coinbase import CoinbaseFetcher
from .coingecko import CoingeckoFetcher
from .cryptowatch import CryptowatchFetcher
from .ftx import FtxFetcher
from .gemini import GeminiFetcher
from .thegraph import TheGraphFetcher
